;;; org.el --- Outline-based notes management and organizer -*- lexical-binding: t; -*-

;; Carstens outline-mode for keeping track of everything.
;; Copyright (C) 2017 Free Software Foundation, Inc.
;;
;; Author: Rasmus Pank Roulund <emacs at pank dot eu>
;; Keywords: outlines, hypermedia, calendar, wp
;; Homepage: http://orgmode.org
;;
;; This file is part of GNU Emacs.
;;
;; GNU Emacs is free software: you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation, either version 3 of the License, or
;; (at your option) any later version.

;; GNU Emacs is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with GNU Emacs.  If not, see <https://www.gnu.org/licenses/>.
;;
;;; Commentary:
;;
;;; Code:

(require 'org)
(require 'tempo)

(defvar org-tempo-tags nil
  "Tempo tags for org-mode")

(defun org-tempo-setup ()
  (tempo-use-tag-list 'org-tempo-tags)
  (setq-local tempo-match-finder "^ *\\(<[[:word:]]\\)\\="))

(add-hook 'org-mode-hook 'org-tempo-setup)

(defun org-tempo-add-tempo (entry)
  "Add entries from `org-structure-template-alist'"
  (let* ((key (format "<%s" (char-to-string (car entry))))
	 (name (cdr entry)))
    (if (assoc-string key org-tempo-tags)
	(setq org-tempo-tags
	      (delete (assoc-string key org-tempo-tags)
		      org-tempo-tags)))
    (tempo-define-template (format "org-%s" (replace-regexp-in-string " " "-" name))
			   `(,(format "#+begin_%s " name) p '> n n
			     ,(format "#+end_%s" (car (org-split-string name " ")))
			     >)
			   key
			   (format "Insert a %s block" name)
			   'org-tempo-tags)))

(mapcar 'org-tempo-add-tempo org-structure-template-alist)

(tempo-define-template "org-include"
		       '("#+include: "
			 (ignore-errors
			   (format "\"%s\" " (file-relative-name (read-file-name "Include file: "))))
			 p >)
		       "<I"
		       "Include keyword"
		       'org-tempo-tags)

(defun org-tempo-insert-space )

(provide 'org-tempo)
;;; org-tempo.el ends here

